Index: apps/app_voicemail.c
===================================================================
--- apps/app_voicemail.c	(revision 348361)
+++ apps/app_voicemail.c	(working copy)
@@ -7549,14 +7549,14 @@
 
 
 
-static int play_message_callerid(struct ast_channel *chan, struct vm_state *vms, char *cid, const char *context, int callback)
+static int play_message_callerid(struct ast_channel *chan, struct vm_state *vms, char *cid, const char *context, int callback, int saycidnumber)
 {
 	int res = 0;
 	int i;
 	char *callerid, *name;
 	char prefile[PATH_MAX] = "";
-	
 
+
 	/* If voicemail cid is not enabled, or we didn't get cid or context from
 	 * the attribute file, leave now.
 	 *
@@ -7582,7 +7582,8 @@
 			if (!res) {
 				snprintf(prefile, sizeof(prefile), "%s%s/%s/greet", VM_SPOOL_DIR, context, callerid);
 				if (!ast_strlen_zero(prefile)) {
-				/* See if we can find a recorded name for this person instead of their extension number */
+					/* See if we can find a recorded name for this callerid
+					 * and if found, use that instead of saying number. */
 					if (ast_fileexists(prefile, NULL, NULL) > 0) {
 						ast_verb(3, "Playing envelope info: CID number '%s' matches mailbox number, playing recorded name\n", callerid);
 						if (!callback)
@@ -7599,10 +7600,23 @@
 			}
 		} else if (!res) {
 			ast_debug(1, "VM-CID: Numeric caller id: (%s)\n", callerid);
-			/* Since this is all nicely figured out, why not say "from phone number" in this case? */
-			if (!callback)
-				res = wait_file2(chan, vms, "vm-from-phonenumber");
-			res = ast_say_digit_str(chan, callerid, AST_DIGIT_ANY, chan->language);
+			/* If there is a recording for this numeric callerid then play that */
+			if (!callback) {
+				/* See if we can find a recorded name for this person instead of their extension number */
+				snprintf(prefile, sizeof(prefile), "%s/Callerids/%s", VM_SPOOL_DIR, callerid);
+				if ( !saycidnumber && ast_fileexists(prefile, NULL, NULL) > 0 ) {
+					ast_verb(3, "Playing recorded name for CID number '%s' - '%s'\n", callerid,prefile);
+					res = wait_file2(chan, vms, "vm-from");
+					res = ast_stream_and_wait(chan, prefile, "");
+					ast_verb(3, "Played recorded name result '%d'\n", res);
+				} else {
+					/* Since this is all nicely figured out, why not say "from phone number" in this case" */
+					res = wait_file2(chan, vms, "vm-from-phonenumber");
+					res = ast_say_digit_str(chan, callerid, AST_DIGIT_ANY, chan->language);
+				}
+			} else {
+				res = ast_say_digit_str(chan, callerid, AST_DIGIT_ANY, chan->language);
+			}
 		}
 	} else {
 		/* Number unknown */
@@ -7787,7 +7801,7 @@
 		res = play_message_datetime(chan, vmu, origtime, filename);
 	}
 	if ((!res) && (ast_test_flag(vmu, VM_SAYCID))) {
-		res = play_message_callerid(chan, vms, cid, context, 0);
+		res = play_message_callerid(chan, vms, cid, context, 0, 0);
 	}
 	if ((!res) && (ast_test_flag(vmu, VM_SAYDURATION))) {
 		res = play_message_duration(chan, vms, duration, vmu->saydurationm);
@@ -13228,10 +13242,12 @@
 		context = ast_variable_retrieve(msg_cfg, "message", "macrocontext");
 	switch (option) {
 	case 3: /* Play message envelope */
-		if (!res)
+		if (!res) {
 			res = play_message_datetime(chan, vmu, origtime, filename);
-		if (!res)
-			res = play_message_callerid(chan, vms, cid, context, 0);
+		}
+		if (!res) {
+			res = play_message_callerid(chan, vms, cid, context, 0, 1);
+		}
 
 		res = 't';
 		break;
@@ -13291,7 +13307,7 @@
 					ast_verb(3, "Confirm CID number '%s' is number to use for callback\n", num);
 					res = ast_play_and_wait(chan, "vm-num-i-have");
 					if (!res)
-						res = play_message_callerid(chan, vms, num, vmu->context, 1);
+						res = play_message_callerid(chan, vms, num, vmu->context, 1, 1);
 					if (!res)
 						res = ast_play_and_wait(chan, "vm-tocallnum");
 					/* Only prompt for a caller-specified number if there is a dialout context specified */